/**
 * @class Apemansleep
 */

'use strict'

class Apemansleep {
  constructor (config = {}) {
    const s = this
    Object.assign(s, config)
    s._timers = {}
    s.sleep = s.sleep.bind(s)
    s.cancel = s.cancel.bind(s)
    s.cancelAll = s.cancelAll.bind(s)
  }

  /**
   * Sleep for duration.
   * @param {number} duration - Sleep milliseconds
   * @returns {Promise.<number>}
   */
  sleep (duration) {
    const s = this
    let timer = null
    let promise = new Promise((resolve) => {
      timer = setTimeout(() => {
        if (!s._timers) {
          return
        }
        delete s._timers[ timer ]
        resolve(timer)
      }, duration)
    })
    promise.cancel = () => s.cancel(timer)
    return promise
  }

  /**
   * Cancel a sleep
   * @param {number} timer - Timeout
   * @returns {Promise.<number>}
   */
  cancel (timer) {
    const s = this
    delete s._timers[ timer ]
    return Promise.resolve(timer)
  }

  /**
   * Cancel all sleeps
   * @returns {Promise.<number[]>}
   */
  cancelAll () {
    const s = this
    let timers = Object.keys(s._timers)
    return Promise.all(
      timers.map((timer) => s.cancel(timer))
    )
  }
}

Object.assign(Apemansleep, {
  create (config) {
    return new Apemansleep(config)
  }
})

module.exports = Apemansleep
